@@ -1,31 +1,17 @@
 /*
- * Copyright (C) 2014-2015 David Rosca <nowrep@gmail.com>
+ * SPDX-FileCopyrightText: 2014-2015 David Rosca <nowrep@gmail.com>
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) version 3, or any
- * later version accepted by the membership of KDE e.V. (or its
- * successor approved by the membership of KDE e.V.), which shall
- * act as a proxy defined in Section 6 of version 3 of the license.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ * SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
  */
 
 #include "adaptertest.h"
 #include "autotests.h"
 #include "device.h"
-#include "pendingcall.h"
 #include "initmanagerjob.h"
+#include "pendingcall.h"
 
-#include <QTest>
 #include <QSignalSpy>
+#include <QTest>
 
 namespace BluezQt
 {
@@ -103,19 +89,13 @@ void AdapterTest::initTestCase()
     initJob->exec();
     QVERIFY(!initJob->error());
 
-    Q_FOREACH (AdapterPtr adapter, m_manager->adapters()) {
+    for (AdapterPtr adapter : m_manager->adapters()) {
         QVERIFY(!adapter->ubi().isEmpty());
 
         AdapterUnit u;
         u.adapter = adapter;
-        u.dbusAdapter = new org::bluez::Adapter1(service,
-                                                 adapter->ubi(),
-                                                 connection,
-                                                 this);
-        u.dbusProperties = new org::freedesktop::DBus::Properties(service,
-                                                                  adapter->ubi(),
-                                                                  connection,
-                                                                  this);
+        u.dbusAdapter = new org::bluez::Adapter1(service, adapter->ubi(), connection, this);
+        u.dbusProperties = new org::freedesktop::DBus::Properties(service, adapter->ubi(), connection, this);
         m_units.append(u);
     }
 
@@ -124,7 +104,7 @@ void AdapterTest::initTestCase()
 
 void AdapterTest::cleanupTestCase()
 {
-    Q_FOREACH (const AdapterUnit &unit, m_units) {
+    for (const AdapterUnit &unit : m_units) {
         delete unit.dbusAdapter;
         delete unit.dbusProperties;
     }
@@ -145,7 +125,7 @@ static void compareUuids(const QStringList &actual, const QStringList &expected)
 
 void AdapterTest::getPropertiesTest()
 {
-    Q_FOREACH (const AdapterUnit &unit, m_units) {
+    for (const AdapterUnit &unit : m_units) {
         QCOMPARE(unit.adapter->ubi(), unit.dbusAdapter->path());
         QCOMPARE(unit.adapter->address(), unit.dbusAdapter->address());
         QCOMPARE(unit.adapter->name(), unit.dbusAdapter->alias());
@@ -165,9 +145,9 @@ void AdapterTest::getPropertiesTest()
 
 void AdapterTest::setAliasTest()
 {
-    Q_FOREACH (const AdapterUnit &unit, m_units) {
+    for (const AdapterUnit &unit : m_units) {
         QSignalSpy adapterSpy(unit.adapter.data(), SIGNAL(nameChanged(QString)));
-        QSignalSpy dbusSpy(unit.dbusProperties, SIGNAL(PropertiesChanged(QString,QVariantMap,QStringList)));
+        QSignalSpy dbusSpy(unit.dbusProperties, SIGNAL(PropertiesChanged(QString, QVariantMap, QStringList)));
 
         QString value = unit.adapter->name() + QLatin1String("_tst_alias");
 
@@ -185,9 +165,9 @@ void AdapterTest::setAliasTest()
 
 void AdapterTest::setPoweredTest()
 {
-    Q_FOREACH (const AdapterUnit &unit, m_units) {
+    for (const AdapterUnit &unit : m_units) {
         QSignalSpy adapterSpy(unit.adapter.data(), SIGNAL(poweredChanged(bool)));
-        QSignalSpy dbusSpy(unit.dbusProperties, SIGNAL(PropertiesChanged(QString,QVariantMap,QStringList)));
+        QSignalSpy dbusSpy(unit.dbusProperties, SIGNAL(PropertiesChanged(QString, QVariantMap, QStringList)));
 
         bool value = !unit.adapter->isPowered();
 
@@ -209,11 +189,11 @@ void AdapterTest::setDiscoverableTest()
 {
     // Discoverable cannot be changed when Adapter is off
 
-    Q_FOREACH (const AdapterUnit &unit, m_units) {
+    for (const AdapterUnit &unit : m_units) {
         unit.adapter->setPowered(true)->waitForFinished();
 
         QSignalSpy adapterSpy(unit.adapter.data(), SIGNAL(discoverableChanged(bool)));
-        QSignalSpy dbusSpy(unit.dbusProperties, SIGNAL(PropertiesChanged(QString,QVariantMap,QStringList)));
+        QSignalSpy dbusSpy(unit.dbusProperties, SIGNAL(PropertiesChanged(QString, QVariantMap, QStringList)));
 
         bool value = !unit.adapter->isDiscoverable();
 
@@ -231,9 +211,9 @@ void AdapterTest::setDiscoverableTest()
 
 void AdapterTest::setDiscoverableTimeoutTest()
 {
-    Q_FOREACH (const AdapterUnit &unit, m_units) {
+    for (const AdapterUnit &unit : m_units) {
         QSignalSpy adapterSpy(unit.adapter.data(), SIGNAL(discoverableTimeoutChanged(quint32)));
-        QSignalSpy dbusSpy(unit.dbusProperties, SIGNAL(PropertiesChanged(QString,QVariantMap,QStringList)));
+        QSignalSpy dbusSpy(unit.dbusProperties, SIGNAL(PropertiesChanged(QString, QVariantMap, QStringList)));
 
         quint32 value = unit.adapter->discoverableTimeout() + 1;
 
@@ -253,9 +233,9 @@ void AdapterTest::setDiscoverableTimeoutTest()
 
 void AdapterTest::setPairableTest()
 {
-    Q_FOREACH (const AdapterUnit &unit, m_units) {
+    for (const AdapterUnit &unit : m_units) {
         QSignalSpy adapterSpy(unit.adapter.data(), SIGNAL(pairableChanged(bool)));
-        QSignalSpy dbusSpy(unit.dbusProperties, SIGNAL(PropertiesChanged(QString,QVariantMap,QStringList)));
+        QSignalSpy dbusSpy(unit.dbusProperties, SIGNAL(PropertiesChanged(QString, QVariantMap, QStringList)));
 
         bool value = !unit.adapter->isPairable();
 
@@ -273,9 +253,9 @@ void AdapterTest::setPairableTest()
 
 void AdapterTest::setPairableTimeoutTest()
 {
-    Q_FOREACH (const AdapterUnit &unit, m_units) {
+    for (const AdapterUnit &unit : m_units) {
         QSignalSpy adapterSpy(unit.adapter.data(), SIGNAL(pairableTimeoutChanged(quint32)));
-        QSignalSpy dbusSpy(unit.dbusProperties, SIGNAL(PropertiesChanged(QString,QVariantMap,QStringList)));
+        QSignalSpy dbusSpy(unit.dbusProperties, SIGNAL(PropertiesChanged(QString, QVariantMap, QStringList)));
 
         quint32 value = unit.adapter->pairableTimeout() + 1;
 
@@ -297,7 +277,7 @@ void AdapterTest::discoveryTest()
 {
     // Discovery needs Adapter powered on
 
-    Q_FOREACH (const AdapterUnit &unit, m_units) {
+    for (const AdapterUnit &unit : m_units) {
         // Make sure the Adapter is powered on and not discovering
         unit.adapter->setPowered(true)->waitForFinished();
         if (unit.adapter->isDiscovering()) {
@@ -305,7 +285,7 @@ void AdapterTest::discoveryTest()
         }
 
         QSignalSpy adapterSpy(unit.adapter.data(), SIGNAL(discoveringChanged(bool)));
-        QSignalSpy dbusSpy(unit.dbusProperties, SIGNAL(PropertiesChanged(QString,QVariantMap,QStringList)));
+        QSignalSpy dbusSpy(unit.dbusProperties, SIGNAL(PropertiesChanged(QString, QVariantMap, QStringList)));
 
         // Start Discovery
         unit.adapter->startDiscovery();
@@ -340,7 +320,7 @@ void AdapterTest::discoveryTest()
 
 void AdapterTest::removeDeviceTest()
 {
-    Q_FOREACH (const AdapterUnit &unit, m_units) {
+    for (const AdapterUnit &unit : m_units) {
         while (!unit.adapter->devices().isEmpty()) {
             DevicePtr device = unit.adapter->devices().first();
 
@@ -361,9 +341,93 @@ void AdapterTest::removeDeviceTest()
     }
 }
 
+static bool isFilterValid(const QStringList &filters, const QVariantMap &filter)
+{
+    auto i = filter.constBegin();
+    while (i != filter.constEnd()) {
+        if (!filters.contains(i.key())) {
+            return false;
+        }
+        i++;
+    }
+    return true;
+}
+
+void AdapterTest::discoveryFilterTest_data() {
+    QTest::addColumn<QVariantMap>("filter");
+    QTest::addColumn<bool>("isValid");
+    QTest::addColumn<bool>("shouldBeDiscoverable");
+
+
+    QTest::newRow("valid-non-discoverable") << QVariantMap({
+        { QStringLiteral("UUIDs"), QStringList() },
+        { QStringLiteral("RSSI"), QVariant::fromValue(qint16(-100)) },
+        { QStringLiteral("Transport"), QStringLiteral("auto") },
+        { QStringLiteral("DuplicateData"), true },
+        { QStringLiteral("Discoverable"), false },
+        { QStringLiteral("Pattern"), QLatin1String("")}
+    }) << true << false;
+
+    QTest::newRow("valid-discoverable") << QVariantMap({
+        { QStringLiteral("UUIDs"), QStringList() },
+        { QStringLiteral("RSSI"), QVariant::fromValue(qint16(-100)) },
+        { QStringLiteral("Transport"), QStringLiteral("auto") },
+        { QStringLiteral("DuplicateData"), true },
+        { QStringLiteral("Discoverable"), true },
+        { QStringLiteral("Pattern"), QLatin1String("")}
+    }) << true << true;
+
+    QTest::newRow("invalid") << QVariantMap({
+        { QStringLiteral("SomeKey"), QStringList() }
+    }) << false << false;
+}
+
+void AdapterTest::discoveryFilterTest()
+{
+    QFETCH(QVariantMap, filter);
+    QFETCH(bool, isValid);
+    QFETCH(bool, shouldBeDiscoverable);
+
+    for (const AdapterUnit &unit : m_units) {
+
+        // Get available discovery filters
+        PendingCall* p = unit.adapter->getDiscoveryFilters(); p->waitForFinished();
+        Q_ASSERT(p->isFinished());
+        QCOMPARE(p->error(), PendingCall::NoError);
+        const QStringList filters = p->value().toStringList();
+
+        // Verify filter
+        QCOMPARE(isFilterValid(filters, filter), isValid);
+
+        // Make sure adapter is powered and not discoverable
+        unit.adapter->setPowered(true)->waitForFinished();
+        unit.adapter->setDiscoverable(false);
+        QTRY_COMPARE(unit.adapter->isDiscoverable(), false);
+        QTRY_COMPARE(unit.dbusAdapter->discoverable(), false);
+
+        QSignalSpy adapterSpy(unit.adapter.data(), SIGNAL(discoverableChanged(bool)));
+        QSignalSpy dbusSpy(unit.dbusProperties, SIGNAL(PropertiesChanged(QString, QVariantMap, QStringList)));
+
+        // Set discovery filter
+        unit.adapter->setDiscoveryFilter(filter);
+
+        if (shouldBeDiscoverable) {
+            // Check if adapter became discoverable
+            QTRY_COMPARE(adapterSpy.count(), 1);
+
+            const QList<QVariant> adapterArguments = adapterSpy.takeFirst();
+            QCOMPARE(adapterArguments.at(0).toBool(), true);
+            Autotests::verifyPropertiesChangedSignal(dbusSpy, QStringLiteral("Discoverable"), true);
+
+            QCOMPARE(unit.adapter->isDiscoverable(), true);
+            QCOMPARE(unit.dbusAdapter->discoverable(), true);
+        }
+    }
+}
+
 void AdapterTest::adapterRemovedTest()
 {
-    Q_FOREACH (const AdapterUnit &unit, m_units) {
+    for (const AdapterUnit &unit : m_units) {
         QSignalSpy managerSpy(m_manager, SIGNAL(adapterRemoved(AdapterPtr)));
         QSignalSpy adapterSpy(unit.adapter.data(), SIGNAL(adapterRemoved(AdapterPtr)));
 
@@ -380,3 +444,5 @@ void AdapterTest::adapterRemovedTest()
 }
 
 QTEST_MAIN(AdapterTest)
+
+#include "moc_adaptertest.cpp"
